class PanelPicture < ActiveRecord::Base
  belongs_to :panel
  belongs_to :picture
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :picture_id, :numericality => true, :existence => {:both => false}
  validates :link, :length => {:maximum => 200}, :url => {:allow_blank => true, :message => I18n.t('errors.messages.url')}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :width, :presence => true, :numericality => true, :not_zero => true, :reverse => true, :resize => true, :sync_vh => true
  validates :height, :presence => true, :numericality => true, :not_zero => true, :reverse => true, :resize => true, :sync_vh => true
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.panel.own?(roles)
    self.panel.visible? roles
  end
  
  def flip
    res = (self.height > 0 ? '' : 'v') + (self.width > 0 ? '' : 'h')
    res += '/' unless res.empty?
    res
  end
  
  def filename
    self.flip + self.picture.filename
  end
  
  def url
    '/pictures/' + self.filename
  end
  
  def opt_img_tag
    {:src => self.url, :vPicture => self.id, :width => self.width.abs, :height => self.height.abs, :alt => self.caption, :style => "top:#{self.y}px; left:#{self.x}px; z-index:#{self.z}; "}
  end
  
  def tmb_opt_img_tag
    tw, th = PettanImager.thumbnail_size(self.width.abs, self.height.abs)
    {:src => self.url, :width => tw, :height => th, :alt => self.caption}
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(PanelPicture.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'panel_pictures.updated_at desc'})
    PanelPicture.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:panel => {:author => {}}, :picture => {:artist => {}, :license => {}}}}
  end
  
  def self.list_json_opt
    {:include => {:panel => {:include => {:author => {}}}, :picture => {:include => {:artist => {}, :license => {}}} }}
  end
  
  def self.mylist au, page = 1, page_size = Author.default_panel_picture_page_size
    opt = {}
    opt.merge!(PanelPicture.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['panels.author_id = ?', au.id], :order => 'panel_pictures.updated_at desc'})
    PanelPicture.find(:all, opt)
  end
  
  def self.show cid, au
    opt = {}
    opt.merge!(PanelPicture.show_opt)
    res = PanelPicture.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}}, :picture => {:artist => {}, :license => {}}}}
  end
  
  def self.show_json_opt
    {:include => {:panel => {:include => {:author => {}}}, :picture => {:include => {:artist => {}, :license => {}}} }}
  end
  
end
